<?php
Namespace App\Controller\Admin\Inventory;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints as Assert;
use App\Model\Admin\HomegearDevice;
use App\Model\Admin\HomegearRoom;
use App\Model\Admin\HomegearGroup;
use App\Model\Admin\HomegearHistory;

class DeviceController extends Controller
{
    public function indexAction(Request $request, HomegearRoom $room, HomegearGroup $group, HomegearDevice $devices)
    {
        return $this->render('Admin/Inventory/Device/list.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'devices' => $devices->loadAll(),
            'rooms' => $room->loadAllForDevice(),
            'groups' => $group->loadAllForDevice()
        ));
    }

    public function addAction(Request $request, HomegearDevice $devices)
    {
        return $this->render('Admin/Inventory/Device/add.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'families' => $devices->listAllKnownFamilies()
        ));
    }

    public function getGatewaysForFamilyAction(Request $request, HomegearDevice $devices)
    {
        $params = $request->request->all();
        $items = $devices->getGatewaysForFamily($params['family']);
        return new JsonResponse($items);
    }

    public function getDevicesForFamilyAction(Request $request, HomegearDevice $devices)
    {
        $params = $request->request->all();
        $items = $devices->getDevicesForFamily($params['family']);
        return new JsonResponse($items);
    }

    public function startSniffingAction(Request $request, HomegearDevice $devices)
    {
        $params = $request->request->all();
        $devices->startSniffing($params['family']);
        return new JsonResponse();
    }

    public function stopSniffingAction(Request $request, HomegearDevice $devices)
    {
        $params = $request->request->all();
        $result = $devices->stopSniffing($params['family']);
        return new JsonResponse($result);
    }

    public function editAction($id, Request $request, HomegearDevice $device, HomegearRoom $room, HomegearGroup $group, HomegearHistory $history)
    {
        return $this->render('Admin/Inventory/Device/edit.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'device' => $device->load($id),
            'rooms' => $room->loadAllForDevice($id),
            'groups' => $group->loadAllForDevice($id),
            'hasInflux' => $history->checkInfluxDb(),
            'history' => $history->getMeasurementsByDevice($id),
            'historyGroups' => $history->loadGroups()
        ));
    }

    public function editConfigParamsAction($id, Request $request, HomegearDevice $device)
    {
        return $this->render('Admin/Inventory/Device/edit_configparams.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'device' => $device->loadConfigParams($id)
        ));
    }

    public function forceConfigUpdateAction($id, Request $request, HomegearDevice $device)
    {
        $device->forceConfigUpdate($id);
        return new JsonResponse();
    }

    public function editHeatingProgramsAction($id, Request $request, HomegearDevice $device)
    {
        return $this->render('Admin/Inventory/Device/edit_heatingprograms.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'device' => $device->loadHeatingProgram($id),
            'templates' => $device->loadHeatingProgramTemplates($id)
        ));
    }

    public function saveHeatingProgramsDeviceAction($id, Request $request, HomegearDevice $device)
    {
        $params = $request->request->all();
        $result = $device->saveHeatingProgramDevice($id,json_decode($params['data'],true));
        return new JsonResponse($result);
    }

    public function getHeatingProgramsTemplateAction($id, Request $request, HomegearDevice $device)
    {
        return new JsonResponse($device->loadHeatingProgramTemplates($id));
    }

    public function saveHeatingProgramsTemplateAction($id, Request $request, HomegearDevice $device)
    {
        $params = $request->request->all();
        $result = $device->saveHeatingProgramTemplate($id,$params);
        return new JsonResponse();
    }

    public function deleteHeatingProgramsTemplateAction($id, Request $request, HomegearDevice $device)
    {
        $params = $request->request->all();
        $result = $device->deleteHeatingProgramTemplate($id,$params);
        return $this->redirectToRoute('admin.inventory.devices.edit.heatingprograms',array('id' => $id),301);
    }

    public function updateAction($id, Request $request, HomegearDevice $device, HomegearRoom $room, HomegearGroup $group, HomegearHistory $history)
    {
        $params = $request->request->all();
        if (!array_key_exists('rooms', $params)) {
            // set an empty rooms-array even if there is none ... we need this to delete the device from all rooms
            $params['rooms'] = array();
        }
        if (!array_key_exists('groups', $params)) {
            // set an empty groups-array even if there is none ... we need this to delete the device from all groups
            $params['groups'] = array();
        }
        // update the device, the associated rooms and groups
        $device->update($id,$params['config']);
        $room->updateDeviceInRooms($id,$params['rooms']);
        $room->updateChannelsInRooms($id,$params);
        $room->updateVariablesInRooms($id,$params);
        $group->updateDeviceInGroups($id,$params['groups']);
        $group->updateChannelsInGroups($id,$params);
        $group->updateVariablesInGroups($id,$params);
        if (array_key_exists('goback',$params) && $params['goback'] != '0') {
            return $this->redirectToRoute($params['goback']);
        } else {
            return $this->redirectToRoute('admin.inventory.devices.edit',array('id' => $id),301);
        }
    }

    public function updateConfigParamsAction($id, Request $request, HomegearDevice $device)
    {
        $params = $request->request->all();
        if (array_key_exists('channel', $params)) {
            $device->updateConfigParams($id,$params);
        }
        return $this->redirectToRoute('admin.inventory.devices.edit.configparams',array('id' => $id),301);
    }

    public function deleteAction($id, HomegearDevice $device, HomegearHistory $history)
    {
        $history->deleteAllByItem($id);
        $device->delete($id);
        return $this->redirectToRoute('admin.inventory.devices');
    }

    public function addHistoryAction($id, Request $request, HomegearHistory $history)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'channel' => new Assert\NotBlank,
                'variable' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $history->addMeasurement($id,$params['channel'],$params['variable']);
        }
        if (array_key_exists('createGraph',$params) && $params['createGraph'] == 1) {
            // create the graph
            $params['variable'] = str_replace('_','',$params['variable']);
            $history->createGraph('history_'.$id.'_'.$params['channel'].'_'.$params['variable'],$params['name'],$params['datatype'],$params['graphtype'],$params['color']);
            $history->addGraphsToGroup($params['graphgroup'],['history_'.$id.'_'.$params['channel'].'_'.$params['variable']]);
        }
        // TODO: check result and redirect with flash
        return $this->redirectToRoute('admin.inventory.devices.edit',array('id' => $id),301);
    }

    public function deleteHistoryAction($id, Request $request, HomegearHistory $history)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'channel' => new Assert\NotBlank,
            'variable' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $history->deleteMeasurement($id,$params['channel'],$params['variable']);
        }
        // TODO: check result and redirect with flash
        return $this->redirectToRoute('admin.inventory.devices.edit',array('id' => $id),301);
    }

    public function addChannelNameAction($id, Request $request, HomegearDevice $device)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'channel' => new Assert\NotBlank,
            'name' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $device->setChannelName($id,$params['channel'],$params['name']);
        }
        return $this->redirectToRoute('admin.inventory.devices.edit',array('id' => $id),301);
    }

    public function deleteChannelNameAction($id, Request $request, HomegearDevice $device)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'channel' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $device->setChannelName($id,$params['channel'],'');
        }
        return $this->redirectToRoute('admin.inventory.devices.edit',array('id' => $id),301);
    }

    public function changeChannelNameAction($id, Request $request, HomegearDevice $device) {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'channel' => new Assert\NotBlank,
            'name' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $device->setChannelName($id,$params['channel'],$params['name']);
        }
        return $this->redirectToRoute('admin.inventory.devices.edit',array('id' => $id),301);
    }

    public function addChannelAssociationAction($id, Request $request, HomegearRoom $room, HomegearGroup $group)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'channel' => new Assert\NotBlank,
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            if (array_key_exists('room',$params)) {
                $room->addChannel($id,$params);
            }
            if (array_key_exists('groups',$params)) {
                $group->addChannel($id,$params);
            }
        }       
        return $this->redirectToRoute('admin.inventory.devices.edit',array('id' => $id));
    }
    
    public function addVariableAssociationAction($id, Request $request, HomegearRoom $room, HomegearGroup $group)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'fields' => array(
                'channel' => new Assert\NotBlank,
                'variable' => new Assert\NotBlank
            ),
            'allowExtraFields' => true
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            if (array_key_exists('room',$params)) {
                $room->addVariable($id,$params);
            }
            if (array_key_exists('groups',$params)) {
                $group->addVariable($id,$params);
            }
        }
        return $this->redirectToRoute('admin.inventory.devices.edit',array('id' => $id));
    }

    public function pairAction($id, $channel, Request $request, HomegearDevice $devices)
    {
        $result = $devices->pair($id,$channel);
        return new JsonResponse();
    }

    public function unpairAction($id, $channel, Request $request, HomegearDevice $devices)
    {
        $result = $devices->unpair($id,$channel);
        return new JsonResponse();
    }

    public function getformAction(Request $request, HomegearDevice $devices)
    {
        $params = $request->request->all();
        return $this->render('Admin/Inventory/Device/add_device_form.html.twig', array(
            'sect' => $request->attributes->get('sect'),
            'device' => $devices->getDeviceDataForAdding($params['family'],$params['type'],$params['gateway'])
        ));
    }

    public function searchDevicesAction($familyId, Request $request, HomegearDevice $devices)
    {
        $result = $devices->searchDevices($familyId);
        if (isset($result) && $result !== null && $result != '0') {
            $msg['devices'] = $result;
            $msg['result'] = "success";
        } elseif ($result == '0') {
            $msg['result'] = "error";
            $msg['message'] = "No new devices could be found.";
        } else {
            $msg['result'] = "error";
        }
        return new JsonResponse($msg);
    }

    public function createDevicesAction($familyId, $typeId, Request $request, HomegearDevice $devices)
    {
        $params = $request->request->all();
        if (array_key_exists('config',$params) && $params['config'] != '') {
            $result = $devices->createDevice($familyId,$typeId,$params['config']);
        }
        return new JsonResponse($result);
    }

    public function setInstallModeAction($familyId, $typeId, $gatewayId, Request $request, HomegearDevice $devices)
    {
        $devices->setInstallMode($familyId, $typeId, $gatewayId);
        $msg['result'] = "success";
        return new JsonResponse($msg);
    }

    public function getPairingStateAction($familyId, Request $request, HomegearDevice $devices)
    {
        return new JsonResponse($devices->getPairingState($familyId));
    }

}