<?php
Namespace App\Controller\Admin\Settings;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Validator\Validation;
use Symfony\Component\Validator\Constraints as Assert;
use App\Model\Admin\HomegearModule;
use App\Utils\Utils;

class ModuleController extends Controller
{
    public function indexAction(Request $request, HomegearModule $module, Utils $utils)
    {
        return $this->render('Admin/Settings/Module/list.html.twig',array(
            'sect' => $request->attributes->get('sect'),
            'modules' => $module->listInstalled(),
            'mgmt' => $utils->checkManagementRunning(false)
        ));
    }

    public function loadAction($id, Request $request, HomegearModule $module)
    {
        return new JsonResponse($module->load($id));
    }

    public function unloadAction($id, Request $request, HomegearModule $module)
    {
        return new JsonResponse($module->unload($id));
    }

    public function reloadAction($id, Request $request, HomegearModule $module)
    {
        return new JsonResponse($module->reload($id));
    }

    public function licenseAction(Request $request, HomegearModule $module)
    {
        // Validate the input
        $validator = Validation::createValidator();
        $constraint = new Assert\Collection(array(
            'module' => new Assert\NotBlank,
            'family' => new Assert\NotBlank,
            'key' => new Assert\NotBlank
        ));
        $params = $request->request->all();
        $errorList = $validator->validate($params, $constraint);
        if (0 === count($errorList)) {
            $result = $module->checkLicense($params['family'],$params['key'],$params['module']);
            return new JsonResponse(array('result' => $result));
        }
    }
}