<?php
Namespace App\Model\Admin;

use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use App\Model\HomegearGroup as BaseGroup;

/**
 * Provides admin functions for Homegear group management
 *
 */
class HomegearGroup extends BaseGroup
{
    public function __construct(ParameterBagInterface $params)
    {
        parent::__construct($params);
    }

    /**
     * Create a new group
     * 
     * @return int
     */
    public function create($name, $lang = null)
    {
        $data = array();
        if ($lang === null) {
            $data = [$this->defaultLanguage => $name];
        } else {
            $data = [$this->defaultLanguage => $name, $lang => $name];
        }
        try {
            $newId = $this->hg->createCategory($data);
        }
        catch(\Homegear\HomegearException $e) {
            return $e;
        }
        return $newId;
    }

    /**
     * Update a group name
     * 
     * @return none
     */
    public function update($id = null, $name = null)
    {
        if ($id === null || $name === null) { return; }
        if (!is_array($name)) {
            $name = [$this->defaultLanguage => "$name"];
        }
        foreach ($name as $l => $n) {
            if ($name == '') {
                unset($name[$l]);
            }
        }
        // Update name
        $this->hg->updateCategory(intval($id),$name);
    }

    /**
     * Delete a group
     * 
     * @return none
     */
    public function delete($id = null) {
        // delete all devices in group
        $devicesInGroup = $this->hg->getDevicesInCategory(intval($id));
        if (count($devicesInGroup) > 0) {
            foreach($devicesInGroup as $k => $v) {
                $this->hg->removeCategoryFromDevice(intval($v),intval($id));
            }
        }
        // delete device sort data
        $this->hg->deleteData(parent::componentGroupGridSort,$id);
        // delete group
        $this->hg->deleteCategory(intval($id));
    }

    /**
     * Update sorting of groups
     * 
     * @return none
     */
    public function updateSort($id = null, $position = null) {
        if ($id === null || $position === null) { return; }
        $this->hg->setCategoryMetadata(intval($id),['position' => intval($position)]);
    }

    /**
     * Load all groups and check if device is in group
     * 
     * @return array()
     */
    public function loadAllForDevice($id = null)
    {
        $allGroups = $this->loadAll();
        if (is_array($allGroups) && count($allGroups) > 0) {
            foreach ($allGroups as $k => $v) {
                $devicesInGroup = $this->hg->getDevicesInCategory(intval($v['ID']));
                $systemVariablesInGroup = $this->hg->getSystemVariablesInCategory(intval($v['ID']));
                if ($id !== null) {
                    if (in_array($id,$devicesInGroup)) {
                        $allGroups[$k]['inGroup'] = 1;
                    } else {
                        $allGroups[$k]['inGroup'] = 0;
                    }
                } else {
                    $allGroups[$k]['devicesInGroup'] = $devicesInGroup;
                }
                if (in_array($id,$systemVariablesInGroup)) {
                    $allGroups[$k]['systemVariablesInGroup'] = 1;
                } else {
                    $allGroups[$k]['systemVariablesInGroup'] = 0;
                }
            }
        }
        return $allGroups;
    }

    /**
     * Update devices in groups
     * 
     * @return none
     */
    public function updateDeviceInGroups($deviceId = null, $groups = array())
    {
        if ($deviceId === null) { return; }
        $allGroups = $this->loadAll();
        if (is_array($allGroups)) {
            // delete the device from all groups
            foreach ($allGroups as $group) {
                $this->hg->removeCategoryFromDevice(intval($deviceId),intval($group['ID']));
            }
        }
        // add device to group(s)
        if (is_array($groups) && count($groups) > 0) {
            foreach($groups as $group2add) {
                $this->hg->addCategoryToDevice(intval($deviceId),intval($group2add));
            }
        }
    }

    public function updateSysvarInGroups($id = null, $groups = array())
    {
        if ($id === null) { return; }
        $allGroups = $this->loadAll();
        if (is_array($allGroups)) {
            // delete the device from all groups
            foreach ($allGroups as $group) {
                $this->hg->removeCategoryFromSystemVariable($id,intval($group['ID']));
                if (count($groups) > 0 && array_search($group['ID'],$groups) !== false) {
                    $this->hg->addCategoryToSystemVariable($id,intval($group['ID']));
                }
            }
        }
    }

    /**
     * Updates channels for a specific device in groups
     */
    public function updateChannelsInGroups($deviceId = null, $data = array())
    {
        if ($deviceId === null || count($data) < 1) { return; }
        $allGroups = $this->loadAll();
        if (is_array($allGroups)) {
            // delete the channels of the device from all groups first
            foreach ($allGroups as $group) {
                $result = $this->hg->getChannelsInCategory($group['ID']);
                if (array_key_exists($deviceId,$result)) {
                    foreach ($result[$deviceId] as $tmp => $channel) {
                        // channel with id '-1' means a complete device, ignore this here!
                        if ($channel == -1) { continue; }
                        $this->hg->removeCategoryFromChannel(intval($deviceId),intval($channel),intval($group['ID']));
                    }
                }
            }
            // now add the POSTed channels again
            if (array_key_exists('channel_groups',$data)) {
                foreach ($data['channel_groups'] as $channel => $data) {
                    foreach ($data as $key => $newCategory) {
                        if ($newCategory != '') {
                            $this->hg->addCategoryToChannel(intval($deviceId),intval($channel),intval($newCategory));
                        }
                    }
                }
            }
        }
    }

    public function updateVariablesInGroups($deviceId = null, $data = array())
    {
        if ($deviceId === null || count($data) < 1) { return; }
        $allGroups = $this->loadAll();
        if (is_array($allGroups)) {
            // delete the channels of the device from all groups first
            foreach ($allGroups as $group) {
                $result = $this->hg->getVariablesInCategory(intval($group['ID']));
                if (array_key_exists($deviceId,$result)) {
                    foreach ($result[$deviceId] as $channel => $values) {
                        foreach ($values as $key => $variable) {
                            $this->hg->removeCategoryFromVariable(intval($deviceId),intval($channel),$variable,intval($group['ID']));
                        }
                    }
                }
            }
            // now add the POSTed variables again
            if (array_key_exists('variable_groups',$data)) {
                foreach ($data['variable_groups'] as $channel => $values) {
                    foreach($values as $variable => $newCategories) {
                        foreach ($newCategories as $key => $newCategory) {
                            if ($newCategory != ''){
                                $this->hg->addCategoryToVariable(intval($deviceId),intval($channel),$variable,intval($newCategory));
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Adds devices to a group
     * 
     * @return none
     */
    public function addDevice($id = null, $devices = null) {
        if ($id === null || $devices === null)  { return; }
        if (is_array($devices) && (count($devices) > 0)) {
            foreach ($devices as $device) {
                $this->hg->addCategoryToDevice(intval($device),intval($id));
            }
        }
    }

    /**
     * Add a channel to a group
     * 
     * @return none
     */
    public function addChannel($id = null, $data = array())
    {
        if ($id === null || count($data) < 1 || !array_key_exists('channel',$data) || !array_key_exists('groups',$data)) { return; }
        if (!is_array($data['groups'])) { return; }
        foreach ($data['groups'] as $group) {
            $this->hg->addCategoryToChannel(intval($id),intval($data['channel']),intval($group));
        }
    }

    /**
     * Add a variable to a group
     * 
     * @return none
     */
    public function addVariable($id = null, $data = array())
    {
        if ($id === null || count($data) < 1 || !array_key_exists('channel',$data) || !array_key_exists('variable',$data) || !array_key_exists('groups',$data)) { return; }
        if (!is_array($data['groups'])) { return; }
        foreach ($data['groups'] as $group) {
            $this->hg->addCategoryToVariable(intval($id),intval($data['channel']),$data['variable'],intval($group));
        }
    }

    public function removeDevice($id = null, $deviceId = null, $channel = null, $variable = null) {
        if ($id === null || $deviceId === null)  { return; }
        // get sorting of devices in current room
        $sort = json_decode($this->hg->getData(parent::componentGroupGridSort,"$id"),true);
        // check if target-device is in sorting-array
        if (is_array($sort) && count($sort) > 0) {
            $key = array_search("$deviceId-$channel-$variable",$sort);
        } else { $key = false; }
        if ($key !== false) {
            // target-device found, delete it from array and save again
            unset($sort[$key]);
            $this->hg->setData(parent::componentGroupGridSort,$id,json_encode($sort));
        }
        if ($deviceId == 'systemVariable') {
            $this->hg->removeCategoryFromSystemVariable($variable,intval($id));
            return;
        }
        if ($channel == '' || $channel == -1) {
            $this->hg->removeCategoryFromDevice(intval($deviceId),intval($id));
            return;
        }
        if ($channel != '' && $channel != -1 && $variable == '') {
            $this->hg->removeCategoryFromChannel(intval($deviceId),intval($channel),intval($id));
            return;
        }
        if ($channel != '' && $channel != -1 && $variable != '') {
            $this->hg->removeCategoryFromVariable(intval($deviceId),intval($channel),$variable,intval($id));
            return;
        }
    }

    public function updateDeviceSort($id = null, $device = null, $position = null, $channel = null, $variable = null) {
        if ($id === null || $device === null || $position === null) { return; }
        // get current sort-data
        $sort = json_decode($this->hg->getData(parent::componentGroupGridSort,"$id"),true);
        // check if target-device is in array already
        if (is_array($sort)) {
            $key = array_search("$device-$channel-$variable",$sort);
            if ($key !== false) {
                // target-device found, delete it from array
                unset($sort[$key]);
            }
        }
        // set new position
        $sort[$position] = "$device-$channel-$variable";
        // save new sort-data
        $this->hg->setData(parent::componentGroupGridSort,$id,json_encode($sort));
    }

}