<?php
Namespace App\Model\Admin;

use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use App\Model\HomegearRoom as BaseRoom;

/**
 * Provides admin functions for Homegear room management
 *
 */
class HomegearRoom extends BaseRoom
{
    public function __construct(ParameterBagInterface $params)
    {
        parent::__construct($params);
    }

    /**
     * Create a new room
     * 
     * @return int
     */
    public function create($name, $lang = null)
    {
        $data = array();
        if ($lang === null) {
            $data = [$this->defaultLanguage => $name];
        } else {
            $data = [$this->defaultLanguage => $name, $lang => $name];
        }
        try {
            $newId = $this->hg->createRoom($data);
        }
        catch(\Homegear\HomegearException $e) {
            return $e;
        }
        return $newId;
    }

    /**
     * Update a room name
     * 
     * @return none
     */
    public function update($id = null, $name = null)
    {
        if ($id === null || $name === null) { return; }
        if (!is_array($name)) {
            $name = [$this->defaultLanguage => "$name"];
        }
        foreach ($name as $l => $n) {
            if ($name == '') {
                unset($name[$l]);
            }
        }
        // Update name
        $this->hg->updateRoom(intval($id),$name);
    }

    /**
     * Delete a room
     * 
     * @return none
     */
    public function delete($id) {
        // delete all devices in room
        $devicesInRoom = $this->hg->getDevicesInRoom(intval($id));
        if (count($devicesInRoom) > 0) {
            foreach($devicesInRoom as $k => $v) {
                $this->hg->removeDeviceFromRoom(intval($v),intval($id));
            }
        }
        $this->hg->deleteData(parent::componentRoomGridSort,$id);
        // delete room
        $this->hg->deleteRoom(intval($id));
    }

    /**
     * Update sorting of rooms
     * 
     * @return none
     */
    public function updateSort($id = null, $position = null) {
        if ($id === null || $position === null) { return; }
        $this->hg->setRoomMetadata(intval($id),['position' => intval($position)]);
    }

    /**
     * Load all rooms and check if device is in room
     * 
     * @return array()
     */
    public function loadAllForDevice($id = null)
    {
        $allRooms = $this->loadAll();
        if (is_array($allRooms) && count($allRooms) > 0 ) {
            foreach ($allRooms as $k => $v) {
                $devicesInRoom = $this->hg->getDevicesInRoom(intval($v['ID']));
                $channelsInRoom = $this->hg->getChannelsInRoom(intval($v['ID']));
                $variablesInRoom = $this->hg->getVariablesInRoom(intval($v['ID']));
                $systemVariablesInRoom = $this->hg->getSystemVariablesInRoom(intval($v['ID']));
                if ($id !== null) {
                    if (in_array($id,$devicesInRoom)) {
                        $allRooms[$k]['deviceInRoom'] = 1;
                    } else {
                        $allRooms[$k]['deviceInRoom'] = 0;
                    }
                } else {
                    $allRooms[$k]['devicesInRoom']  = $devicesInRoom;
                }
                if (in_array($id,$channelsInRoom)) {
                    $allRooms[$k]['channelInRoom'] = 1;
                } else {
                    $allRooms[$k]['channelInRoom'] = 0;
                }
                if (in_array($id,$variablesInRoom)) {
                    $allRooms[$k]['variableInRoom'] = 1;
                } else {
                    $allRooms[$k]['variableInRoom'] = 0;
                }
                if (in_array($id,$systemVariablesInRoom)) {
                    $allRooms[$k]['systemVariablesInRoom'] = 1;
                } else {
                    $allRooms[$k]['systemVariablesInRoom'] = 0;
                }
            }
        }
        return $allRooms;
    }

    /**
     * Update devices in rooms
     * 
     * @return none
     */
    public function updateDeviceInRooms($deviceId, $rooms = array())
    {
        $allRooms = $this->loadAll();
        if (is_array($allRooms)) {
            // delete the device from all rooms
            foreach ($allRooms as $room) {
                $this->hg->removeDeviceFromRoom(intval($deviceId),intval($room['ID']));
            }
            if (count($rooms) > 0 && $rooms[0] != '') {
                foreach ($rooms as $key => $newRoom) {
                    $this->hg->addDeviceToRoom(intval($deviceId),intval($newRoom));
                }
            }
        }
    }

    public function updateSysvarInRooms($id = null, $rooms = array())
    {
        $allRooms = $this->loadAll();
        if (is_array($allRooms)) {
            // delete the sysvar from all rooms
            foreach ($allRooms as $room) {
                $this->hg->removeSystemVariableFromRoom($id,intval($room['ID']));
            }
            if (count($rooms) > 0 && $rooms[0] != '') {
                foreach ($rooms as $key => $newRoom) {
                    $this->hg->addSystemVariableToRoom($id,intval($newRoom));
                }
            }
        }
    }

    /**
     * Update channel(s) for a specific device in rooms
     * 
     * @return none
     */
    public function updateChannelsInRooms($deviceId = null, $data = array())
    {
        if ($deviceId === null || count($data) < 1) { return; }
        $allRooms = $this->loadAll();
        if (is_array($allRooms)) {
            // delete the channels of the device from all rooms first
            foreach ($allRooms as $room) {
                $result = $this->hg->getChannelsInRoom(intval($room['ID']));
                if (array_key_exists($deviceId,$result)) {
                    foreach ($result[$deviceId] as $tmp => $channel) {
                        // -1 means a whole device
                        if ($channel == -1) { continue; }
                        $this->hg->removeChannelFromRoom(intval($deviceId),intval($channel),intval($room['ID']));
                    }
                }
            }
            // now add the POSTed channels again
            if (array_key_exists('channel_room',$data)) {
                foreach ($data['channel_room'] as $channel => $newRoom) {
                    if ($newRoom != '') {
                        $this->hg->addChannelToRoom(intval($deviceId),intval($channel),intval($newRoom));
                    }
                }
            }
        }
    }

    /**
     * Update variable(s) for a specific device in rooms
     * 
     * @return none
     */
    public function updateVariablesInRooms($deviceId = null, $data = array())
    {
        if ($deviceId === null || count($data) < 1) { return; }
        $allRooms = $this->loadAll();
        if (is_array($allRooms)) {
            // delete the variables of the device from all rooms first
            foreach ($allRooms as $room) {
                $result = $this->hg->getVariablesInRoom(intval($room));
                if (array_key_exists($deviceId,$result)) {
                    foreach ($result[$deviceId] as $channel => $values) {
                        // -1 means a whole device
                        if ($channel == -1) { continue; }
                        foreach ($values as $key => $variable) {
                            $this->hg->removeVariableFromRoom(intval($deviceId),intval($channel),$variable,intval($room));
                        }
                    }
                }
            }
            // now add the POSTed variables again
            if (array_key_exists('variable_room',$data)) {
                foreach ($data['variable_room'] as $channel => $values) {
                    foreach($values as $variable => $newRoom) {
                        if ($newRoom != '') {
                            $this->hg->addVariableToRoom(intval($deviceId),intval($channel),$variable,intval($newRoom));
                        }
                    }
                }
            }
        }
    }

    /**
     * Adds devices to a room
     * 
     * @return none
     */
    public function addDevice($id = null, $devices = null) {
        if ($id === null || $devices === null)  { return; }
        if (is_array($devices) && (count($devices) > 0)) {
            foreach ($devices as $device) {
                $this->hg->addDeviceToRoom(intval($device),intval($id));
            }
        }
    }

    /**
     * Add a channel to a room
     * 
     * @return none
     */
    public function addChannel($id = null, $data = array())
    {
        if ($id === null || count($data) < 1 || !array_key_exists('channel',$data) || !array_key_exists('room',$data)) { return; }
        $this->hg->addChannelToRoom(intval($id),intval($data['channel']),intval($data['room']));
    }

    /**
     * Add a variable to a room
     * 
     * @return none
     */
    public function addVariable($id = null, $data = array())
    {
        if ($id === null || count($data) < 1 || !array_key_exists('channel',$data) || !array_key_exists('variable',$data) || !array_key_exists('room',$data)) { return; }
        $this->hg->addVariableToRoom(intval($id),intval($data['channel']),$data['variable'],intval($data['room']));
    }

    public function removeDevice($id = null, $deviceId = null, $channel = null, $variable = null) {
        if ($id === null || $deviceId === null)  { return; }
        // get sorting of devices in current room
        $sort = json_decode($this->hg->getData(parent::componentRoomGridSort,"$id"),true);
        // check if target-device is in sorting-array
        if (is_array($sort) && count($sort) > 0) {
            $key = array_search("$deviceId-$channel-$variable",$sort);
        } else { $key = false; }
        if ($key !== false) {
            // target-device found, delete it from array and save again
            unset($sort[$key]);
            $this->hg->setData(parent::componentRoomGridSort,$id,json_encode($sort));
        }
        if ($deviceId == 'systemVariable') {
            $this->hg->removeSystemVariableFromRoom($variable,intval($id));
            return;
        }
        if ($channel == '' || $channel == -1) {
            $this->hg->removeDeviceFromRoom(intval($deviceId),intval($id));
            return;
        }
        if ($channel != '' && $channel != -1 && $variable == '') {
            $this->hg->removeChannelFromRoom(intval($deviceId),intval($channel),intval($id));
            return;
        }
        if ($channel != '' && $channel != -1 && $variable != '') {
            $this->hg->removeVariableFromRoom(intval($deviceId),intval($channel),$variable,intval($id));
            return;
        }
    }

    public function updateDeviceSort($id = null, $device = null, $position = null, $channel = null, $variable = null) {
        if ($id === null || $device === null || $position === null) { return; }
        // get current sort-data
        $sort = json_decode($this->hg->getData(parent::componentRoomGridSort,"$id"),true);
        // check if target-device is in array already
        if (is_array($sort)) {
            $key = array_search("$device-$channel-$variable",$sort);
            if ($key !== false) {
                // target-device found, delete it from array
                unset($sort[$key]);
            }
        }
        // set new position
        $sort[$position] = "$device-$channel-$variable";
        // save new sort-data
        $this->hg->setData(parent::componentRoomGridSort,$id,json_encode($sort));
    }
}