<?php
Namespace App\Model\Admin;

use App\Model\HomegearVariable as BaseVariable;

/**
 * Provides admin functions for Homegear variable management
 *
 */
class HomegearVariable extends BaseVariable
{
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Create a system variable
     * 
     * @return int
     */
    public function create($name = null, $type = null, $flag = null, $value = null) {
        if ($name === null || $type === null) { return; }
        if ($flag === null) { $flag = 0; } // visible, readwrite
        switch ($type) {
            case "string":
                if ($value == null) { $value = ''; }
                $result = $this->hg->setSystemVariable($name,"$value", intval($flag));
            break;
            case "int":
                if ($value == null) { $value = 1; }
                $result = $this->hg->setSystemVariable($name,intval($value), intval($flag));
            break;
            case "float":
                if ($value == null) { $value = 0.12; }
                $result = $this->hg->setSystemVariable($name,floatval($value), intval($flag));
            break;
            case "bool":
                if ($value == null) { $value = 1; }
                $result = $this->hg->setSystemVariable($name, filter_var($value, FILTER_VALIDATE_BOOLEAN), intval($flag));
            break;
            case "array":
                if ($value == null) { $value = []; }
                $result = $this->hg->setSystemVariable($name, $value, intval($flag));
            break;
            case "struct":
                if ($value == null) { $value = [[]]; }
                $result = $this->hg->setSystemVariable($name, $value, intval($flag));
            break;
        }
        /*if ($result == -32700) {
            return $this->outputMessage("Data could not be encoded to binary format.",0);
        } elseif ($result == -32602) {
            return $this->outputMessage("Name is longer than 250 characters.",0);
        } elseif ($result == -32500) {
            return $this->outputMessage("The limit of 1 million metadata entries has been reached.",0);
        }*/
        return $result;
    }

    /**
     * Load a specific system variable
     * 
     * @return array()
     */
    public function loadByName($name = null) {
        if ($name === null) { return; }
        $v = $this->hg->getSystemVariable($name);
        $flag = $this->hg->getSystemVariableFlags($name);
        $type = 'struct';
        $value = print_r($v, true);
        if (is_string($v)) { $type = 'string'; $value = $v; }
        if (is_int($v)) { $type = 'int'; $value = intval($v); }
        if (is_float($v)) { $type = 'float'; $value = floatval($v); }
        if (is_bool($v)) { $type = 'bool'; $value = boolval($v); }
        if (is_array($v)) { $type = 'array'; $value = print_r($v, true); }
        return ['name' => $name, 'url' => base64_encode($name), 'type' => $type, 'flag' => ($flag ? $flag : 0), 'value' => $value];
    }

    /**
     * Delete a system variable
     * 
     * @return none
     */
    public function delete($name = null) {
        if ($name === null) { return; }
        $this->hg->deleteSystemVariable($name);
    }

}