<?php
Namespace App\Model;

/**
 * Provides base functions for Homegear history and graph data
 *
 */
class HomegearHistory
{
    const componentGroupGridSort = 'sortGroupsGrid';
    const componentGraphGroups = 'GraphGroups';
    const componentGraphConfig = 'configGraphs';
    public $hg;
    private $measurements;

    public function __construct()
    {
        $this->hg = new \Homegear\Homegear;
    }

    public function checkInfluxDb()
    {
        if (call_user_func("Homegear\\Homegear::system.listMethods", "influxdbGetDatabase")) {
            return true;
        }
        return false;
    }

    /**
     * Load all known measurements and group by un-/configured
     * 
     * @return array()
     */
    public function loadMeasurements()
    {
        $data = array();
        // get all measurements from influx
        if ($this->measurements === null) { $this->_getAllMeasurements(); }
        if (is_array($this->measurements) && count($this->measurements) > 0) {
            foreach ($this->measurements as $graph) {
                // try to load graph config for measurement
                $config = $this->getGraphConfig($graph[0]);
                if ($config === null) {
                    $graph = $this->_formatRawMeasurement($graph[0]);
                    if (array_key_exists('orphaned',$graph)) {
                        $data['orphaned'][] = $graph;
                    } else {
                        $data['unconfigured'][] = $graph;
                    }
                } else {
                    if (array_key_exists('orphaned',$graph)) {
                        $data['orphaned'][] = $graph;
                    } else {
                        $data['configured'][] = $config;
                    }
                }
            }
        }
        if (isset($data['configured'])) {
            usort($data['configured'], function($a, $b) { return $a['sort'] <=> $b['sort']; });
        }
        return $data;
    }

    public function getGraphConfig($graph = null)
    {
        if ($graph === null) { return; }
        $config = $this->hg->getData(self::componentGraphConfig,$graph);
        if ($config !== null) {
            $deviceData = $this->_formatRawMeasurement($graph);
            $config = json_decode($config,true);
            $config['id'] = $graph;
            $config['deviceId'] = $deviceData['deviceId'];
            $config['deviceName'] = $deviceData['deviceName'];
            return $config;
        }
    }

    public function _formatRawMeasurement($data = null) {
        if ($data === null) { return; }
        $graph = explode('_',$data);
        if ($graph[1] == 0 && $graph[2] == 'n') {
            // this is a system variable
            $device = ['ID' => 0, 'NAME' => 'System Variable'];
        } else {
            try {
                $device = $this->hg->getAllConfig(intval($graph[1]))[0];
            } catch(\Homegear\HomegearException $e) {
                return ['id' => $data, 'deviceId' => $graph[1], 'deviceName' => 'Device not found (deleted?)', 'channel' => $graph[2], 'variable' => $graph[3], 'orphaned' => true];
            }
        }
        return ['id' => $data, 'deviceId' => $device['ID'], 'deviceName' => ($device['NAME'] != '' ? $device['NAME'] : $device['ADDRESS']), 'channel' => $graph[2], 'variable' => $graph[3]];
    }

    /**
     * Load all measurements from influxdb
     * 
     * @return array()
     */
    private function _getAllMeasurements()
    {
        try {
            $all = $this->hg->influxdbQuery(false, "SHOW MEASUREMENTS");
        } catch(\Homegear\HomegearException $e) {
            return;
        }
        if (isset($all['results'][0]['series'])) {
            $this->measurements = $all['results'][0]['series'][0]['values'];
        }
    }

}