<?php
namespace App\Utils;

use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Translation\TranslatorInterface;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\HttpFoundation\Session\Session;
use \Homegear\Homegear as Homegear;

Class Utils
{
    protected $hg;
    protected $translator;
    protected $request;
    protected $router;

    public function __construct(RequestStack $requestStack, TranslatorInterface $translator, RouterInterface $router)
    {
        $this->hg = new Homegear;
        $this->translator = $translator;
        $this->request = $requestStack;
        $this->router = $router;
    }

    public function checkManagementRunning($fullstatus = true)
    {
        if (call_user_func("Homegear\\Homegear::system.listMethods", "managementGetCommandStatus")) {
            return true;
        }
        if (!$fullstatus) { return false; }
        $current = $this->request->getCurrentRequest()->get('_route');
        return $result = ['status' => '500', 'message' => $this->translator->trans('management.not_running',array(),'admin'), 'callback' => "setTimeout(checkMgmt, 5000, '".$this->router->generate('check', ['type' => 'mgmt'])."', '".$this->router->generate($current)."');"];
    }

    public function checkWebSshRunning($fullstatus = true)
    {
        if (call_user_func("Homegear\\Homegear::system.listMethods", "websshInput")) {
            return true;
        }
        if (!$fullstatus) { return false; }
        $current = $this->request->getCurrentRequest()->get('_route');
        return $result = ['status' => '500', 'message' => $this->translator->trans('webssh.not_running',array(),'admin'), 'callback' => "setTimeout(checkWebSsh, 5000, '".$this->router->generate('check', ['type' => 'webssh'])."', '".$this->router->generate($current)."');"];
    }

    public function checkCa()
    {
        if ($this->checkManagementRunning(false)) {
            return $this->hg->managementCaExists();
        }
        return -1;
    }

    public function checkReadonly()
    {
        $result = shell_exec('grep "\sro[\s,]" /proc/mounts |grep "\s/\s"');
        if (is_null($result)) { return false; }
        return true;
    }

    public function getCommandStatus($id) {
        if ($this->checkManagementRunning(false)) {
            $result = $this->hg->managementGetCommandStatus($id);
            while (!$result['finished']) {
                usleep(100000); // 100ms
                $result = $this->hg->managementGetCommandStatus($id);
            }
            return $result;
        }
        return 0;
    }

    public function getDataPath()
    {
        $conf = file_get_contents('/etc/homegear/main.conf');
        preg_match('/^writeableDataPath = (.+)/m',$conf,$result);
        if (!is_array($result) || count($result) < 1) {
            preg_match('/^dataPath = (.+)/m',$conf,$result);
        }
        return $result[1];
    }

    public function saveToSession($data = false)
    {
        if ($data === false || !is_array($data)) { return; }
        $session = new Session();
        foreach ($data as $key => $value) {
            $session->set("$key", "$value");
        }
    }

    public function checkUpdateRunning($type = false)
    {
        if ($type === false) { return false; }
        $session = new Session();
        $update = $session->get($type);
        if ($update) {
            if ($this->checkManagementRunning(false)) {
                try {
                    $result = $this->hg->managementGetCommandStatus(intval($update));
                } catch(\Homegear\HomegearException $e) {
                    return false;
                }
                if ($result && $result['finished'] !== true) {
                    return true;
                }
                $session->remove($type);
                if ($type == '_os-update-running') {
                    unlink("/tmp/updateSystem.txt");
                }
                if ($type == '_sys-update-running') {
                    unlink("/tmp/updateHomegear.txt");
                }
                return false;
            }
            return -1;
        }
        return -1;
    }

    public function getIpAddresses($hg, $int = false) {
        $tmp = shell_exec('ifconfig');
        $tmp = preg_split("/\n\n/",$tmp);
        foreach ($tmp as $k => $v) {
            if ($v == '') { continue; }
            // TODO: this is not reliable!
            $interface = trim(substr($v, 0, strpos($v, ': flags')));
            if ($interface == '') {
                $interface = trim(substr($v, 0, strpos($v, 'Link')));
            }
            if (strpos($interface,'lo') === false) {
                preg_match('/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/', $v, $ip);
                if (($int !== false) && ($int == $interface) && array_key_exists(0,$ip)) {
                    return $ip[0];
                }
                if (array_key_exists(0,$ip)) {
                    $interfaces[$k]['name'] = $interface;
                    $interfaces[$k]['ip'] = $ip[0];
                }
            }
        }
        return array_values($interfaces);
    }

    public function getHost()
    {
        $host = $_SERVER['HTTP_HOST'];
        if ($_SERVER['SERVER_ADDR'] != $host) {
            if (stristr($host,':')) {
                list($host,$port) = explode(':',$host);
            }
            return $host;
        }
        return $_SERVER['HTTP_HOST'];
    }

    public function getPort()
    {
        $host = $_SERVER['HTTP_HOST'];
        if (array_key_exists('HTTPS',$_SERVER) && $_SERVER['HTTPS'] == 'on') {
            $port = '443';
        } else {
            $port = '80';
        }
        if (ip2long($host) === false) {
            if (stristr($host,':')) {
                list($host,$port) = explode(':',$host);
            }
        }
        return $port;
    }

    public function checkExpert($username = null)
    {
        if ($username == null) { return false; }
        $userSettings = $this->hg->getUserMetadata($username);
        if (array_key_exists('expert', $userSettings) && $userSettings['expert'] == "true") {
            return true;
        }
        return false;
    }

    public function checkOss()
    {
        if (substr($_SERVER['REQUEST_URI'],0,6) !== '/admin') {
            return false;
        }
        return true;
    }

    public function checkHGS()
    {
        return defined('HGS');
    }
}